﻿using System;
using System.Collections.Generic;
using System.Text;

namespace ROS.Smtp
{
    /// <summary>
    /// SMTPコマンドと、内部的に利用するコマンドのstaticクラス
    /// </summary>
    /// <remarks>
    /// <para>
    /// 基本的に使うものしか入っていないので、RFCに定義されているもの全てがあるわけではないので注意すること。
    /// </para>
    /// </remarks>
    public static class SmtpCommands
    {
        //public constしかない
        #region Data / SMTPコマンドのDATA
        /// <summary>
        /// SMTPコマンドのDATA
        /// </summary>
        /// <remarks>
        /// <para>
        /// メールのデータを送信するためのコマンドで、MAIL, RCPTの後に使われる。 
        /// DATAコマンドに対してパラメータはなく、サーバが受理するならコマンドそのものに対してまず354応答が返される。
        /// これが返されたら、クライアントは DATAの内容(つまりメールメッセージの内容：ヘッダとボディ)を送信する。
        /// ここで1行は78文字(全角なら39文字)以内であるべきで、998文字以内でなければならない(RFC 2822)。
        /// 送信してよいデータは7ビットASCIIすべて。ただし、スペース、CRLF以外は送信すると誤動作するかもしれない。
        /// また、「8bitMIME」で拡張されることもある。 
        /// </para>
        /// <para>
        /// データ(メッセージ)のおわりは「.」(ピリオド)だけの行で示される。
        /// 本当に「.」だけのデータを送信したい場合のため、送信側(クライアント)は「.」で始まる行の先頭にもう1つの「.」をつけくわえ、サーバは行頭の「.」を取り除く。
        /// つまり、「.forward」とだけ書いた行は「..forward」として送信され、サーバで元に戻される。
        /// 「.」だけの行なら「..」として送信される。 
        /// </para>
        /// <note>
        /// 面倒くさいので、この処理はない。
        /// </note>
        /// <para>
        /// データの終わりに対し、サーバは成功(250)か失敗を返す。
        /// 成功が返されたときにサーバはそのメッセージを配信したり、リレーしたり、キューに入れたりする。
        /// この時点でクライアントはこのメッセージに関する一切の責任を、サーバに渡したことになる(つまり、ここで成功が返された後には再送してはならない)。 
        /// </para>
        /// <note>
        /// メッセージを配信する機能は実装する予定もない。
        /// </note>
        /// </remarks>
        public const string Data = "DATA"; 
        #endregion

        #region HELO / SMTPコマンドのHELO
        /// <summary>
        /// SMTPコマンドのHELO
        /// </summary>
        /// <remarks>
        /// <para>
        /// HELOにより、SMTPサーバはSMTPクライアントを識別し、もしそのクライアントがそのSMTPサーバを利用できるならばセッションが開始される。
        /// これに対して、 SMTPサーバは応答コード250でサーバ自身の名前(FQDN)を返す(グリーティング)。 
        /// </para>
        /// <para>
        /// SMTPクライアントのFQDNが存在しない場合には、そのIPアドレスが送られ、(もし存在するなら)続いてそのクライアントを識別するのに有用な情報が送られる。 
        /// </para>
        /// <note>
        /// FQDNは存在しないので、IPしか返さない。
        /// </note>
        /// <para>
        /// 現在では、このコマンドではなくEHLOを利用すべきであり、HELOはEHLOを理解できない古いクライアント、サーバのためにのみ存在する(が、サーバはHELOをサポートしなければならない)。 
        /// </para>
        /// </remarks>
        public const string HELO = "HELO";
        #endregion

        #region EHLO / SMTPコマンドのEHLO
        /// <summary>
        /// SMTPコマンドのEHLO
        /// </summary>
        /// <remarks>
        /// <para>
        /// EHLOにより、SMTPサーバはSMTPクライアントを識別し、もしそのクライアントがそのSMTPサーバを利用できるならばセッションが開始される。
        /// これに対して、 SMTPサーバは応答コード250で以下の情報(普通複数行)を返す(グリーティング)。
        /// なお、SMTPサーバは、実際にはTCP/IP接続のIPアドレスなどの情報からクライアントを識別すべきである。 
        /// </para>
        /// <list type="list">
        /// <item>サーバ自身の名前(FQDN) </item>
        /// <item>このサーバで利用できる拡張サービス</item>
        /// </list>
        /// <para>
        /// クライアントは、基本コマンドに加えて、ここでサーバから返される拡張コマンドを利用することもできる。
        /// それ以外を利用してはならない。
        /// なお、ここで表示されるのは、拡張サービスの名称やコマンド名ではなく、拡張サービスに関連したEHLOキーワードと呼ばれるものである (拡張サービス表にはEHLOキーワードと表記)。 
        /// </para>
        /// <para>
        /// SMTPクライアントのFQDNが存在しない場合には、そのIPアドレスが送られ、(もし存在するなら)続いてそのクライアントを識別するのに有用な情報が送られる。 
        /// </para>
        /// <note>
        /// 拡張は何もない（というか、拡張すらない）ので、何もデータは追加で送られない。
        /// </note>
        /// </remarks>
        public const string EHLO = "EHLO";
        #endregion

        #region Mail / SMTPコマンドのMAIL
        /// <summary>
        /// SMTPコマンドのMAIL
        /// </summary>
        /// <remarks>
        /// <para>
        /// このメールの(エンベロープ上：つまり、配送システム上)の送り主を示す。
        /// メールヘッダのFrom: (RFC 2822で定義されている)とは意味が違う点に注意。
        /// EHLO(HELO)がメールセッション(クライアントとサーバの接続に関する処理)の初期化をするのに対し、メールトランザクション(ある1つのメールについての処理)を初期化する(このコマンド以前にサーバに渡された宛て先やデータは消去される)。
        /// メールトランザクションが進行している時にはMAILコマンドは利用できない。普通、EHLOコマンド直後に使われる。 
        /// </para>
        /// <note>
        /// EHLOをサポートしていないので、当然のことながら、拡張はない。
        /// </note>
        /// <para>
        /// (アドレス)は送り主のメールボックスアドレス(を「&gt;&lt;」で括ったもの)である。現在ではホストのリストを前置するフォーマット(例えば puni!shiori!mimori のような)は使うべきではない。
        /// 一部のエラーメールなどについて(アドレス)が「&gt;&lt;」のみの場合がある(メールがループするのを防ぐため)。 
        /// </para>
        /// <note>
        /// オプションはEHLO でサーバからそれに関連するキーワードが返された場合にのみ利用可能なので、そもそもEHLOがないので、オプションなんてない。
        /// </note>
        /// </remarks>
        public const string Mail = "MAIL"; 
        #endregion

        #region Noop / SMTPコマンドのNOOP
        /// <summary>
        /// SMTPコマンドのNOOP
        /// </summary>
        /// <remarks>
        /// <para>
        /// 何もしないコマンド。パラメータがあっても動作は同じ。
        /// </para>
        /// </remarks>
        public const string Noop = "NOOP";
        #endregion

        #region Quit / SMTPコマンドのQUIT
        /// <summary>
        /// SMTPコマンドのQUIT
        /// </summary>
        /// <remarks>
        /// <para>
        /// メールセッションを終了する。これに対して221の応答を受信したら、コネクションを閉じる。 
        /// </para>
        /// </remarks>
        public const string Quit = "QUIT"; 
        #endregion

        #region Rcpt / SMTPコマンドのRCPT
        /// <summary>
        /// SMTPコマンドのRCPT
        /// </summary>
        /// <remarks>
        /// <para>
        /// メールをどこに送るのか、という受信者を指定するコマンド。
        /// メールヘッダのTo:やCc:など(RFC 2822で定義されている)とは意味が違う点に注意。
        /// 同じメッセージを複数の宛て先に送る場合、1回のメールトランザクションでこのコマンドを複数回使うことができる。
        /// なお、MAILコマンドより後にしか利用できない。 
        /// </para>
        /// <para>
        /// 宛て先のアドレスに普通のもの(mimori@puni.netなど)とする他、「@iyadesu.org:mimori@puni.net 」というように書けば iyadesu.org を経由 (リレー)してmimori@puni.net へとメールが送信される(ただし、 iyadesu.org がメールリレーを許可していなければ、550エラーとなってしまう)。 
        /// </para>
        /// <note>
        /// むろん、リレーなどしないので、550を返すべきだが、この子は何でもかんでも受信するので、550を返したりはしない。
        /// </note>
        /// <note>
        /// オプションはEHLO でサーバからそれに関連するキーワードが返された場合にのみ利用可能なので、当然オプションはない。
        /// </note>
        /// </remarks>
        public const string Rcpt = "RCPT"; 
        #endregion

        #region Rset / SMTPコマンドのRSET
        /// <summary>
        /// SMTPコマンドのRSET
        /// </summary>
        /// <remarks>
        /// <para>
        /// このコマンドにより、現在のメールトランザクションを中止する。
        /// サーバは、ここまでに送られた送り元、送り先、データなどすべてを廃棄する。
        /// 簡単にいえば、EHLO(HELO)直後の状態へ戻る(EHLOを送信してもほとんど同じ結果が得られるが、EHLOよりもRSETの方が単純なコマンドである)。このコマンドは常に成功し、250応答が返る。 
        /// </para>
        /// </remarks>
        public const string Rset = "RSET"; 
        #endregion

        #region Vrfy / SMTPコマンドのVRFY
        /// <summary>
        /// SMTPコマンドのVRFY
        /// </summary>
        /// <remarks>
        /// <para>
        /// (アドレス or ユーザ名)が存在するかどうかを確認する。
        /// 実際にそのサーバ上のメールボックスが存在することを確認できたら、250の応答コードでユーザのメールボックス名(と、オプションでユーザのフルネームなど)が返される。
        /// メールボックスが別サーバにあるなどの理由で実際には確認できないが、文法上は正しいなら252が返される。また、その引数で複数のアドレスにマッチするなどの場合には551が返る。 
        /// </para>
        /// <para>
        /// もしVRFYしたアドレスがメーリングリストの場合には、EXPNと同じように振る舞う実装もあり、一方ではエラーとする実装もある(いずれも正しい実装である)。 
        /// </para>
        /// <para>
        /// 最近では、セキュリティ上の理由によりこのコマンドを利用できないようにしているサーバが多く、その場合には550が返る。
        /// </para>
        /// <note>
        /// というか、550しか返さない。
        /// </note>
        /// </remarks>
        public const string Vrfy = "VRFY"; 
        #endregion


        //コマンドではないもの
        #region CrLf / CRLF に相当する文字列。\r\n
        /// <summary>
        /// CRLF に相当する文字列。\r\n
        /// </summary>
        public const string CrLf = "\r\n"; 
        #endregion

        #region DataEnd / SMTP的に最後のデータとする文字列 . + CrLf
        /// <summary>
        /// SMTP的に最後のデータとする文字列 . + CrLf
        /// </summary>
        public const string DataEnd = "." + CrLf;
        #endregion

        #region DataFollow / SMTPのDATAを受け取った後の内部的な状態。DATAを受信中とする状態とする
        /// <summary>
        /// SMTPのDATAを受け取った後の内部的な状態。DATAを受信中とする状態とする
        /// </summary>
        public const string DataFollow = "_DATAFOLLOW";
        #endregion

        #region RequestAccepted / SMTPのDATAが終了した事を知るための内部的な状態。
        /// <summary>
        /// SMTPのDATAが終了した事を知るための内部的な状態。
        /// </summary>
        public const string RequestAccepted = "_REQEST_ACCEPTED";
        #endregion
    }
}
